"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EFFECTIVE_TARGET_PROP = void 0;
exports["default"] = AOPProxy;

var _handlerFactory = _interopRequireDefault(require("./factory/handlerFactory"));

var _isAOPProxy = require("./isAOPProxy");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

var EFFECTIVE_TARGET_PROP = "-FKgGSKjfL00OFSs8fX5S";
/**
 * Generates a new AOP proxy object.
 * An AOP proxy object allows proxying and advising for the following operations:
 * - Calling a method (`call()` method);
 * - Getting a property (`get()` method);
 * - Setting a property (`set()` method);
 * - Calling a function (`[applyRule]` symbol property with `apply()` method);
 * - Constructing a new object via the "new" keyword (`[constructRule]` symbol
 * property with `construct()`).
 * @param {*} target A function, a constructor function or an object to proxy.
 * @param {Object|Array} proxyRules The proxy rules.
 * This parameter can be:
 * - An object containing proxy rules as keys and pointcuts with advices as values;
 * - An array of rules, each rule either being an object with proxy rules as keys
 * and advices as values or a tuple of two elements: an array of rules of the same
 * type or a single rule as the first element and a pointcut with advices for those rules or
 * that rule as the second element;
 * @return {Proxy} A new proxy object for the given target.
 */

exports.EFFECTIVE_TARGET_PROP = EFFECTIVE_TARGET_PROP;

function AOPProxy(target, proxyRules) {
  var handler = (0, _handlerFactory["default"])(proxyRules);
  var proxy = new Proxy(target, handler);
  var effectiveTarget = target[_isAOPProxy.isAOPProxy] ? target[EFFECTIVE_TARGET_PROP] : target;
  proxy[EFFECTIVE_TARGET_PROP] = effectiveTarget;
  return proxy;
}
//# sourceMappingURL=AOPProxy.js.map