"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = POJOPromiser;

var _AOPProxy = _interopRequireDefault(require("../AOPProxy"));

var _get = _interopRequireDefault(require("../pointcuts/shorthands/get"));

var _functional = require("../../misc/functional");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * @type {Symbol}
 */
var queueTupleProp = Symbol("queueTupleProp");
/**
 * @type {Symbol}
 */

var dataProp = Symbol("dataProp");
/**
 * Builtin function using AOPProxy's API to create a POJO promiser,
 * i.e. a proxy object with syntactic sugar which lets creating a queue of properties to resolve
 * and resolves to an object with those same properties and their respective resolved values.
 * @see https://gist.github.com/jasuperior/5d339f9c2572b3bb52d487de4086a3b2
 * @param {Object} obj A POJO object where each property refers to a value to use as is or to
 * a promise resolving to the value to use for that property.
 * @param {Object} options Options.
 * @param {boolean} [options.mutateTargetOnSettledPromiseQueue] Whether or not to mutate the
 * original given "obj" POJO object with the resolved values or not (defaults to false,
 * i.e. it does not mutate the original "obj" POJO object).
 * @return {Proxy} A new proxy object for the given target which supports this builtin feature.
 */

function POJOPromiser(obj) {
  var _that;

  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$mutateTargetOnSe = _ref.mutateTargetOnSettledPromiseQueue,
      mutateTargetOnSettledPromiseQueue = _ref$mutateTargetOnSe === void 0 ? false : _ref$mutateTargetOnSe;

  var originalTarget = obj; // This is needed because the Proxy API may be polyfilled with proxy-polyfill (https://github.com/GoogleChrome/proxy-polyfill/).
  // Cloning "obj" allows setting properties on the target clone.

  var target = _objectSpread({}, obj);

  var that = (_that = {}, _defineProperty(_that, queueTupleProp, void 0), _defineProperty(_that, dataProp, {}), _that);

  var rules = _defineProperty({}, "/.?/", (0, _get["default"])().around(function () {
    var prop = this.property,
        target = this.target;

    if (prop === "then" && that[queueTupleProp]) {
      // If "then" is accessed, and there is something in the queue.
      var _that$queueTupleProp = _slicedToArray(that[queueTupleProp], 2),
          queuedProp = _that$queueTupleProp[0],
          queuedPromise = _that$queueTupleProp[1];

      var _promise = queuedPromise.then(function (data) {
        var obj = that[dataProp]; // Resolved data from last promise.

        obj[queuedProp] = data;
        return new Promise(function (resolve) {
          that[dataProp] = {}; // Erase data.

          that[queueTupleProp] = void 0; // Clear queue.

          resolve(obj); // Resolve promise queue.

          if (obj && mutateTargetOnSettledPromiseQueue) {
            // Assign resolved values to the original target, so that
            // properties which value is a promise will be replaced with the resolved value.
            Object.assign(originalTarget, obj);
          }
        });
      });

      var then = _promise.then.bind(_promise);

      return then;
    }

    var promise;

    if (!(target[prop] instanceof Promise) && prop !== "then") {
      // If the prop is not "then" or a promise, i.e. it is a real value.
      if (that[queueTupleProp]) {
        // If there is a queue.
        var _that$queueTupleProp2 = _slicedToArray(that[queueTupleProp], 2),
            _queuedProp = _that$queueTupleProp2[0],
            _queuedPromise = _that$queueTupleProp2[1];

        var _obj = that[dataProp];
        promise = _queuedPromise.then(function (data) {
          // Resolve last promise.
          _obj[_queuedProp] = data;
          return Promise.resolve(target[prop]);
        });
      } else {
        // Otherwise, simply create a new promise with value.
        that[dataProp] = {};
        promise = Promise.resolve(target[prop]);
      }
    }

    if (that[queueTupleProp]) {
      // If it is a promise and there is a queue.
      var _that$queueTupleProp3 = _slicedToArray(that[queueTupleProp], 2),
          _queuedProp2 = _that$queueTupleProp3[0],
          _queuedPromise2 = _that$queueTupleProp3[1];

      var _obj2 = that[dataProp];
      promise = _queuedPromise2.then(function (data) {
        // Resolve last promise.
        _obj2[_queuedProp2] = data;
        return target[prop];
      });
    } else if (!promise) {
      // target[prop] is of course a promise here (invariant).
      that[dataProp] = {};
      promise = target[prop];
    }

    that[queueTupleProp] = [prop, promise]; // Save the promise and the property in the queue.
    // All alternations ensure that no matter what the value, a promise which must be resolved
    // is always given.

    target.then = _functional.noActionFunc;
    var proxyTarget = (0, _AOPProxy["default"])(target, rules);
    return proxyTarget;
  }));

  target.then = _functional.noActionFunc;
  var proxyTarget = (0, _AOPProxy["default"])(target, rules);
  return proxyTarget;
}
//# sourceMappingURL=POJOPromiser.js.map