"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = trapHandlerForRules;

var _rulesGenerator = _interopRequireDefault(require("../rules/rulesGenerator"));

var _parseRule = _interopRequireDefault(require("../rules/parseRule"));

var _Trap = _interopRequireDefault(require("./Trap"));

var _isAOPProxy = require("../isAOPProxy");

var _AOPProxy = require("../AOPProxy");

var _reflect = require("./reflect");

var _objects = require("../../object-handling/objects");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _readOnlyError(name) { throw new TypeError("\"" + name + "\" is read-only"); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Generates a new trap handler object for the given rules.
 *
 * @param {Object|Array} proxyRules The proxy rules.
 * @return {Object} The new trap handler object for the given rules.
 */
function trapHandlerForRules(proxyRules) {
  var trap = new _Trap["default"]();
  var rules = (0, _rulesGenerator["default"])(proxyRules);

  var _iterator = _createForOfIteratorHelper(rules),
      _step;

  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _step$value = _step.value,
          rule = _step$value.rule,
          pointcut = _step$value.pointcut;
      var parsedRule = (0, _parseRule["default"])(rule);
      trap.addPointcutRule(pointcut, parsedRule);
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }

  var privateScope = {};
  return _objectSpread(_objectSpread(_objectSpread(_objectSpread({}, {
    // Trap for property access (getting) and method call.
    get: function get(target, property, receiver) {
      if (property === _isAOPProxy.isAOPProxy) return true;else if (property === _AOPProxy.EFFECTIVE_TARGET_PROP) return privateScope[_AOPProxy.EFFECTIVE_TARGET_PROP];
      var propertyValue = trap.hasGets() ? trap.get(target, property, receiver) : ((0, _reflect.reflectGet)(target, property, receiver), _readOnlyError("propertyValue"));
      return propertyValue;
    }
  }), {
    // Trap for property access (setting).
    set: function set(target, property, value, receiver) {
      if (property === _AOPProxy.EFFECTIVE_TARGET_PROP) {
        (0, _objects.defineProperty)(privateScope, _AOPProxy.EFFECTIVE_TARGET_PROP, {
          value: value
        });
        return true;
      }

      var updateWasSuccessful = trap.hasSets() ? trap.set(target, property, value, receiver) : (0, _reflect.reflectSet)(target, property, value, receiver);
      return updateWasSuccessful;
    }
  }), trap.hasApplies() ? {
    // Trap for function call.
    apply: function apply(target, thisArg, argumentsList) {
      return trap.apply(target, thisArg, argumentsList);
    }
  } : {}), trap.hasConstructs() ? {
    // Trap for object construction with the "new" operator.
    construct: function construct(target, argumentsList, newTarget) {
      return trap.construct(target, argumentsList, newTarget);
    }
  } : {});
}
//# sourceMappingURL=trapHandlerForRules.js.map