"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.lazyMap = void 0;
exports.lazyMapFactory = lazyMapFactory;
exports.map = map;

var _ImmutableLinkedOrderedMap = require("./ImmutableLinkedOrderedMap");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Shortcut function to create an immutable linked ordered map.
 * @param {Object} options Options.
 * @return {ImmutableLinkedOrderedMap} A new immutable linked ordered map for the given options.
 */
function map() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return new _ImmutableLinkedOrderedMap.ImmutableLinkedOrderedMap(_objectSpread({}, options));
}
/**
 * Shortcut function to create a lazy immutable linked ordered map.
 * @param {Object} options Options.
 * @return {ImmutableLinkedOrderedMap} A new lazy immutable linked ordered map for the given options.
 */


var lazyMap = function lazyMap() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return map(_objectSpread(_objectSpread({}, options), {}, {
    lazy: true
  }));
};
/**
 * Shortcut function returning a factory function to create a lazy map for a given key property name "keyPropName".
 * Example:
 * ```
 * const mapFactory = lazyMapFactory("item_id");
 * let map = mapFactory([{item_id: 1, prop: "a"}, {item_id: 2, prop: "b"}, {item_id: 3, prop: "c"}]);
 * map.get(1); // {item_id: 1, prop: "a"}
 * map = map.set({item_id: 4, prop: "d"}); // Append
 * //map = map.set({item_id: 4, prop: "d"}, true); // Prepend
 * map.get(4); // {item_id: 4, prop: "d"}
 * map.get(5); // undefined
 * map = map.unset(4);
 * map.get(4); // undefined
 * ```
 *
 * @param {string} keyPropName The key property name to use for the items in the map that will be created by the returned factory function.
 * @return {(initialItems: Array) => ImmutableLinkedOrderedMap} A factory function to create a lazy map for the given property name "keyPropName".
 */


exports.lazyMap = lazyMap;

function lazyMapFactory() {
  var keyPropName = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : DEFAULT_KEY_PROP_NAME;
  return function () {
    var initialItems = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    return lazyMap({
      keyPropName: keyPropName,
      initialItems: initialItems
    });
  };
}
//# sourceMappingURL=shortcuts.js.map