"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isValidDate = isValidDate;
exports.isValidDateTimeStr = isValidDateTimeStr;
exports.utcDate = utcDate;

var _strings = require("../misc/strings");

/**
 * Check if a value is a date.
 * @param {*} value The value to test.
 * @return {boolean} True if it is a valid date.
 */
function isValidDate(value) {
  return Object.prototype.toString.call(value) === "[object Date]" && !isNaN(value.getTime());
}
/**
 * Check if a value is a valid datetime string (e.g. in ISO "YYYY-MM-DD HH:ii:ss" format).
 * @param {string} str The string.
 * @return {boolean} True if it is a valid datetime string.
 */


function isValidDateTimeStr(str) {
  return isValidDate(new Date(str));
}
/**
 * Returns a UTC datetime in ISO `YYYY-MM-DD HH:ii:ss` format.
 * @param {Object} [options] An optional object of options.
 * @param {boolean} [options.Y] True to return the year. Default: true.
 * @param {boolean} [options.m] True to return the month. Default: true.
 * @param {boolean} [options.d] True to return the day. Default: true.
 * @param {boolean} [options.H] True to return the hours. Default: true.
 * @param {boolean} [options.i] True to return the minutes. Default: true.
 * @param {boolean} [options.s] True to return the seconds. Default: true.
 * @param {Date|string} [options.date] The date to format. Default: the current date.
 * @return {string} The UTC datetime in ISO `YYYY-MM-DD HH:ii:ss` format.
 */


function utcDate() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [],
      _ref$Y = _ref.Y,
      Y = _ref$Y === void 0 ? true : _ref$Y,
      _ref$m = _ref.m,
      m = _ref$m === void 0 ? true : _ref$m,
      _ref$d = _ref.d,
      d = _ref$d === void 0 ? true : _ref$d,
      _ref$H = _ref.H,
      H = _ref$H === void 0 ? true : _ref$H,
      _ref$i = _ref.i,
      i = _ref$i === void 0 ? true : _ref$i,
      _ref$s = _ref.s,
      s = _ref$s === void 0 ? true : _ref$s,
      _ref$date = _ref.date,
      date = _ref$date === void 0 ? null : _ref$date;

  var finalDate = date ? new Date(date) : new Date();
  var year = finalDate.getUTCFullYear();
  var month = (finalDate.getUTCMonth() + 1 + "").padStart(2, "0");
  var day = (finalDate.getUTCDate() + "").padStart(2, "0");
  var hours = (finalDate.getUTCHours() + "").padStart(2, "0");
  var minutes = (finalDate.getUTCMinutes() + "").padStart(2, "0");
  var seconds = (finalDate.getUTCSeconds() + "").padStart(2, "0");
  var UTCDate = (0, _strings.trim)((0, _strings.trim)("".concat(Y ? year : "", "-").concat(m && (Y || d || !Y && !d) ? month : "", "-").concat(d ? day : ""), " -") + " " + (0, _strings.trim)("".concat(H ? hours : "", ":").concat(i && (H || s || !H && !s) ? minutes : "", ":").concat(s ? seconds : ""), " :"), " ");
  var finalUTCDate = UTCDate.replace("--", "-").replace("::", ":");
  return finalUTCDate;
}
//# sourceMappingURL=date.js.map