"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.msToTime = msToTime;
exports.now = now;
exports.objToMilliseconds = objToMilliseconds;
exports.parseMilliseconds = parseMilliseconds;
exports.prettifyMilliseconds = prettifyMilliseconds;
exports.secsToTimeString = secsToTimeString;
exports.time = time;
exports.timeZone = timeZone;

var _math = require("../maths/math");

var _strings = require("../misc/strings");

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * Returns the current Unix time in milliseconds.
 * @return {number} Current Unix time in milliseconds.
 */
function now() {
  return Date.now() || new Date().getTime();
}
/**
 * Returns the current Unix time in seconds.
 * @return {number} Current Unix time in seconds.
 */


function time() {
  var seconds = Math.round(now() / 1000);
  return seconds;
}
/**
 * Returns the time string approximated to the nearest microsecond
 * corresponding the given Unix timestamp in milliseconds
 * in the format "HH:mm:ss.ms".
 * @param {number} [ms] The time in microseconds to convert into a string.
 * If omitted, the current time will be used.
 * @return {string} The time string.
 */


function msToTime(ms) {
  var date = new Date(ms || now());
  var isoDate = new Date(date.getTime() - date.getTimezoneOffset() * 60000).toISOString();
  return isoDate.slice(11, -1);
}
/**
 * Converts seconds to a `minutes:seconds` or `hours:minutes:seconds` time string
 * (if `includeHours` is true).
 * @example secsToTimeString(123) => "02:03".
 * @example secsToTimeString(123, includeHrs = true) => "00:02:03".
 * @param {number} secs The number of seconds.
 * @param {boolean} includeHrs Optional.
 * @return {string} The time string.
 */


function secsToTimeString(secs) {
  var includeHrs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
  if (!secs || !new RegExp(/^[0-9]+$/).test(secs)) return includeHrs ? "00:00:00" : "00:00";
  var hours = "";
  var minutes = (0, _math.integerDivision)(secs, 60);

  if (includeHrs) {
    hours = "".concat(((0, _math.integerDivision)(secs, 60 * 60) + "").padStart(2, "0"), ":");
    minutes = minutes % 60;
  }

  minutes = "".concat(minutes).padStart(2, "0");
  var seconds = "".concat(secs % 60).padStart(2, "0");
  return "".concat(hours).concat(minutes, ":").concat(seconds);
}
/**
 * Parse milliseconds into an object.
 * @param {number} milliseconds the milliseconds to parse.
 * @returns an object containing days, hrs, mins, secs, ms, µs, ns.
 *  */


function parseMilliseconds(milliseconds) {
  if (typeof milliseconds !== "number") throw new TypeError("Expected a number");
  return {
    days: Math.trunc(milliseconds / 86400000),
    hours: Math.trunc(milliseconds / 3600000) % 24,
    minutes: Math.trunc(milliseconds / 60000) % 60,
    seconds: Math.trunc(milliseconds / 1000) % 60,
    milliseconds: Math.trunc(milliseconds) % 1000,
    microseconds: Math.trunc(milliseconds * 1000) % 1000,
    nanoseconds: Math.trunc(milliseconds * 1e6) % 1000
  };
}
/**
 * Convert milliseconds to a human readable string: 1337000000 → 15d 11h 23m 20s
 * @param {number} ms
 * @param {Object} options
 * @param {number} options.secsDecimalDigits
 * Default: 1. Number of digits to appear after the seconds decimal point.
 * @param {number} options.msDecimalDigits
 * Default: 0. Number of digits to appear after the milliseconds decimal point.
 * Useful in combination with process.hrtime().
 * @param {boolean} options.keepDecimalsOnWholeSeconds
 * Default: false. Keep milliseconds on whole seconds: 13s → 13.0s.
 * Useful when you are showing a number of seconds spent on an operation and don't want
 * the width of the output to change when hitting a whole number.
 * @param {boolean} options.compact
 * Default: false. Only show the first unit: 1h 10m → 1h. Also ensures that
 * millisecondsDecimalDigits and secondsDecimalDigits are both set to 0.
 * @param {number} options.unitCount
 * Default: Infinity. Number of units to show. Setting compact to true overrides this option.
 * @param {boolean} options.verbose
 * Default: false. Use full-length units: 5h 1m 45s → 5 hours 1 minute 45 seconds
 * @param {boolean} options.separateMilliseconds
 * Default: false. Show milliseconds separately. This means they won't be included in the
 * decimal part of the seconds.
 * @param {boolean} options.formatSubMilliseconds
 * Default: false. Show microseconds and nanoseconds.
 * @param {boolean} options.colonNotation
 * Default: false. Display time using colon notation: 5h 1m 45s → 5:01:45. Always shows time
 * in at least minutes: 1s → 0:01. Useful when you want to display time without the time units,
 * similar to a digital watch. Setting colonNotation to true overrides the following options to
 * false: compact, formatSubMilliseconds, separateMilliseconds, verbose
 * @returns the human readable time.
 */


function prettifyMilliseconds(ms) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var SECOND_ROUNDING_EPSILON = 0.0000001;
  if (!Number.isFinite(ms)) throw new TypeError("Expected a finite number");

  if (options.colonNotation) {
    options.compact = false;
    options.formatSubMilliseconds = false;
    options.separateMilliseconds = false;
    options.verbose = false;
  }

  if (options.compact) {
    options.secsDecimalDigits = 0;
    options.msDecimalDigits = 0;
  }

  var result = [];

  var floorDecimals = function floorDecimals(value, decimalDigits) {
    var flooredInterimValue = Math.floor(value * Math.pow(10, decimalDigits) + SECOND_ROUNDING_EPSILON);
    var flooredValue = Math.round(flooredInterimValue) / Math.pow(10, decimalDigits);
    return flooredValue.toFixed(decimalDigits);
  };

  var add = function add(value, _long, _short, valueString) {
    if ((result.length === 0 || !options.colonNotation) && value === 0 && !(options.colonNotation && _short === "m")) return;
    valueString = (valueString || value || "0").toString();
    var prefix;
    var suffix;

    if (options.colonNotation) {
      prefix = result.length > 0 ? ":" : "";
      suffix = "";
      var wholeDigits = valueString.includes(".") ? valueString.split(".")[0].length : valueString.length;
      var minLength = result.length > 0 ? 2 : 1;
      valueString = "0".repeat(Math.max(0, minLength - wholeDigits)) + valueString;
    } else {
      prefix = "";
      suffix = options.verbose ? " " + (0, _strings.pluralize)(_long, value) : _short;
    }

    result.push(prefix + valueString + suffix);
  };

  var parsed = parseMilliseconds(ms);
  add(Math.trunc(parsed.days / 365), "year", "y");
  add(parsed.days % 365, "day", "d");
  add(parsed.hours, "hour", "h");
  add(parsed.minutes, "minute", "m");

  if (options.separateMilliseconds || options.formatSubMilliseconds || !options.colonNotation && ms < 1000) {
    add(parsed.seconds, "second", "s");

    if (options.formatSubMilliseconds) {
      add(parsed.milliseconds, "millisecond", "ms");
      add(parsed.microseconds, "microsecond", "µs");
      add(parsed.nanoseconds, "nanosecond", "ns");
    } else {
      var millisecondsAndBelow = parsed.milliseconds + parsed.microseconds / 1000 + parsed.nanoseconds / 1e6;
      var millisecondsDecimalDigits = typeof options.msDecimalDigits === "number" ? options.msDecimalDigits : 0;
      var roundedMiliseconds = millisecondsAndBelow >= 1 ? Math.round(millisecondsAndBelow) : Math.ceil(millisecondsAndBelow);
      var millisecondsString = millisecondsDecimalDigits ? millisecondsAndBelow.toFixed(millisecondsDecimalDigits) : roundedMiliseconds;
      add(Number.parseFloat(millisecondsString), "millisecond", "ms", millisecondsString);
    }
  } else {
    var seconds = ms / 1000 % 60;
    var secsDecDigits = typeof options.secsDecimalDigits === "number" ? options.secsDecimalDigits : 1;
    var secsFixed = floorDecimals(seconds, secsDecDigits);
    var secsStr = options.keepDecimalsOnWholeSeconds ? secsFixed : secsFixed.replace(/\.0+$/, "");
    add(Number.parseFloat(secsStr), "second", "s", secsStr);
  }

  if (result.length === 0) return "0" + (options.verbose ? " milliseconds" : "ms");
  if (options.compact) return result[0];

  if (typeof options.unitCount === "number") {
    var separator = options.colonNotation ? "" : " ";
    return result.slice(0, Math.max(options.unitCount, 1)).join(separator);
  }

  return options.colonNotation ? result.join("") : result.join(" ");
}

var converters = {
  days: function days(value) {
    return value * 864e5;
  },
  hours: function hours(value) {
    return value * 36e5;
  },
  minutes: function minutes(value) {
    return value * 6e4;
  },
  seconds: function seconds(value) {
    return value * 1e3;
  },
  milliseconds: function milliseconds(value) {
    return value;
  },
  microseconds: function microseconds(value) {
    return value / 1e3;
  },
  nanoseconds: function nanoseconds(value) {
    return value / 1e6;
  }
};
/** Convert an object of time properties to milliseconds: {seconds: 2} → 2000 */

function objToMilliseconds(timeDescriptor) {
  var totalMilliseconds = 0;

  for (var _i = 0, _Object$entries = Object.entries(timeDescriptor); _i < _Object$entries.length; _i++) {
    var _Object$entries$_i = _slicedToArray(_Object$entries[_i], 2),
        key = _Object$entries$_i[0],
        value = _Object$entries$_i[1];

    if (typeof value !== "number") throw new TypeError("Expected a `number` for key `".concat(key, "`, got `").concat(value, "` (").concat(_typeof(value), ")"));
    var converter = converters[key];
    if (!converter) throw new Error("Unsupported time key: ".concat(key));
    totalMilliseconds += converter(value);
  }

  return totalMilliseconds;
}
/**
 * Pretty time zone: +2 or -9:30
 * @param {Date} date
 * @returns A human readable timezone.
 */


function timeZone() {
  var date = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();
  var offset = date.getTimezoneOffset();
  var absOffset = Math.abs(offset);
  var hours = Math.floor(absOffset / 60);
  var minutes = absOffset % 60;
  var minutesOut = minutes > 0 ? ":" + ("0" + minutes).slice(-2) : "";
  return (offset < 0 ? "+" : "-") + hours + minutesOut;
}
//# sourceMappingURL=time.js.map