"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.geoStandardOptions = void 0;
exports.isSamePosition = isSamePosition;
exports.makePositionObj = makePositionObj;

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

var geoStandardOptions = Object.freeze({
  enableHighAccuracy: false,
  timeout: 0xffffffff,
  maximumAge: 0
});
/**
 * Checks if two position are equal. Instead of throwing an error, this method
 * will also return false if the supplied args are invalid.
 * @param {Object} current a Position object.
 * @param {Object} next the obj to compare against.
 * @returns {boolean} True if they are equal.
 */

exports.geoStandardOptions = geoStandardOptions;

function isSamePosition(current, next) {
  if (_typeof(current) !== "object" || _typeof(next) !== "object" || !current || !next || !next.coords || current.timestamp && next.timestamp && current.timestamp !== next.timestamp) return false;
  return current.coords.latitude === next.coords.latitude && current.coords.longitude === next.coords.longitude && current.coords.altitude === next.coords.altitude && current.coords.accuracy === next.coords.accuracy && current.coords.altitudeAccuracy === next.coords.altitudeAccuracy && current.coords.heading === next.coords.heading && current.coords.speed === next.coords.speed;
}
/**
 * Given a object with coords properties returns only its timestamp 
 * and coord properties.
 * @param {Object} position The object to convert.
 * @returns {Object} a position object.
 */


function makePositionObj(position) {
  return !position || _typeof(position) !== 'object' ? null : {
    timestamp: position.timestamp,
    coords: {
      latitude: position.coords.latitude,
      longitude: position.coords.longitude,
      altitude: position.coords.altitude,
      accuracy: position.coords.accuracy,
      altitudeAccuracy: position.coords.altitudeAccuracy,
      heading: position.coords.heading,
      speed: position.coords.speed
    }
  };
}
//# sourceMappingURL=geolocation.js.map