"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.integerDivision = integerDivision;
exports.isEven = isEven;
exports.isOdd = isOdd;
exports.proportion = proportion;
exports.round = round;
exports.sum = sum;

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * Rounds a number.
 * @param {number} number The number to round.
 * @param {number} precision The decimal precision (significant figures).
 * @return {number} The rounded number.
 */
function round(number) {
  var precision = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
  var factor = Math.pow(10, precision);
  return Math.round(number * factor) / factor;
}
/**
 * Sums two or more numbers.
 * @param {...number} numbers The numbers to sum.
 * @return {number} The sum.
 */


function sum() {
  for (var _len = arguments.length, numbers = new Array(_len), _key = 0; _key < _len; _key++) {
    numbers[_key] = arguments[_key];
  }

  return numbers.reduce(function (carry, num) {
    return carry + num;
  });
}
/**
 * Integer division (without decimal part).
 * @param {number} number A number.
 * @param {number} divideBy The number to divide by.
 * @return {number} The result of the integer division.
 */


function integerDivision(number, divideBy) {
  return Math.floor(number / divideBy);
}
/**
 * Check if a number is even.
 * @param {number} num A number.
 * @return {boolean} True if even, false if odd.
 */


function isEven(num) {
  return num % 2 === 0;
}
/**
 * Check if a number is odd.
 * @param {number} num A number.
 * @return {boolean} True if odd, false if even.
 */


function isOdd(num) {
  return !(num % 2 === 0);
}
/**
 * Converts a source value of a source range to a value that is proportional
 * in another destination range.
 * @param {number[]} sourceTuple The source range tuple.
 * This function assumes that both the starting and ending values are positive numbers
 * (the starting value can be 0).
 * @return {(destRange: number[]) => (sourceValue: number) => number}
 * A function receiving the destination range tuple as argument (having the same shape
 * as the source range) returning a function taking the source value as argument returning
 * the value proportional in the destination range.
 */


function proportion(_ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      sourceFrom = _ref2[0],
      sourceTo = _ref2[1];

  return function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        destFrom = _ref4[0],
        destTo = _ref4[1];

    return function (sourceValue) {
      return (sourceValue - sourceFrom) * ((destTo - destFrom) / (sourceTo - sourceFrom)) + destFrom;
    };
  };
}
//# sourceMappingURL=math.js.map