"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mean = mean;
exports.median = median;
exports.minDeviationFrom = minDeviationFrom;
exports.minDeviationFromExcluding = minDeviationFromExcluding;

var _math = require("./math");

var _arrays = require("../object-handling/arrays");

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

/**
 * Computes the mean of the given numbers.
 * @param {...number} numbers The numbers.
 * @return {number} The mean.
 */
function mean() {
  return _math.sum.apply(void 0, arguments) / arguments.length;
}
/**
 * Computes the median of the given numbers.
 * @param {...number} numbers The numbers.
 * @return {number} The median value.
 */


function median() {
  for (var _len = arguments.length, numbers = new Array(_len), _key = 0; _key < _len; _key++) {
    numbers[_key] = arguments[_key];
  }

  var sorted = (0, _arrays.sortNums)(numbers);
  var middleIndex = (0, _math.integerDivision)(sorted.length, 2);
  return (0, _math.isOdd)(sorted.length) ? sorted[middleIndex] : mean(sorted[middleIndex - 1], sorted[middleIndex]);
}
/**
 * Computes the min deviation from a value.
 * @param {number} value The value to deviate from.
 * @return {(...numbers: number[]) => number} A function which takes the numbers
 * to use to compute the minimum deviation from "value".
 */


function minDeviationFrom(value) {
  return function () {
    var minAbsDeviation = Infinity;

    for (var _len2 = arguments.length, numbers = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      numbers[_key2] = arguments[_key2];
    }

    numbers.map(function (num) {
      var deviation = Math.abs(num - value);

      if (deviation < minAbsDeviation) {
        minAbsDeviation = deviation;
      }
    });
    return minAbsDeviation;
  };
}
/**
 * Computes the min deviation from a value excluding that value from the numbers received.
 * @param {number} value The value to exclude and check deviations from.
 * @return {(...numbers: number[]) => number} A function which takes the numbers to use to
 * compute the minimum deviation from value, exluding value.
 */


function minDeviationFromExcluding(value) {
  return function () {
    for (var _len3 = arguments.length, numbers = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      numbers[_key3] = arguments[_key3];
    }

    return minDeviationFrom(value).apply(void 0, _toConsumableArray(numbers.filter(function (num) {
      return num !== value;
    })));
  };
}
//# sourceMappingURL=statistics.js.map