"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.debounce = debounce;
exports.delay = void 0;
exports.throttle = throttle;

/**
 * Delays execution of a callback and cancels a previously registered callback if it wasn't
 * executed yet.
 */
var delay = function () {
  var timer = 0;
  /**
   * Inner function.
   * @param {Function} callback The callback to execute.
   * @param {number} ms Milliseconds to wait before executing the callback.
   * @return {undefined}
   */

  return function (callback, ms) {
    clearTimeout(timer);
    timer = setTimeout(function () {
      callback();
    }, ms);
  };
}();
/**
 * Debounces a function.
 * @param {Function} func A function.
 * @param {number} wait Wait interval in milliseconds.
 * @return {Function} A new function, debounced.
 */


exports.delay = delay;

function debounce(func, wait) {
  var timer = void 0;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    timer !== undefined && clearTimeout(timer);
    timer = setTimeout(function () {
      func.apply(void 0, args);
    }, wait);
  };
}
/**
 * Throttles a function.
 * @see https://www.sitepoint.com/throttle-scroll-events/
 * @param {Function} func A function.
 * @param {number} wait Wait interval in milliseconds.
 * @return {Function} A new function, throttled.
 */


function throttle(func, wait) {
  var time = Date.now();
  return function () {
    if (time + wait - Date.now() < 0) {
      func.apply(void 0, arguments);
      time = Date.now();
    }
  };
}
//# sourceMappingURL=callbacks.js.map