"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports._ = void 0;
exports.chain = chain;
exports.curry = curry;
exports.noActionFunc = noActionFunc;
exports.pick = pick;
exports.proceedCallingFn = void 0;

function _toArray(arr) { return _arrayWithHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _iterableToArray(iter) { if (typeof Symbol !== "undefined" && iter[Symbol.iterator] != null || iter["@@iterator"] != null) return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) return _arrayLikeToArray(arr); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * A higher-order function to create a chain of functions following the Chain of
 * Responsibility design pattern.
 * @param {...Function|...Function[]} funcs A list of functions or higher-order functions
 * or arrays of functions (arrays will be flattened) to chain.
 * @return {Function} A function representing the chain of the given functions which, if called,
 * will return the result of the chain. Each function will receive the next function as its
 * last parameter.
 */
function chain() {
  for (var _len = arguments.length, funcs = new Array(_len), _key = 0; _key < _len; _key++) {
    funcs[_key] = arguments[_key];
  }

  function chainLink(func, next) {
    return function (args) {
      return func(args, next && function () {
        for (var _len2 = arguments.length, args = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        return next(args);
      });
    };
  }

  return function () {
    funcs = funcs.flat(1);
    var chainFunc = funcs.reduceRight(function (nextChainLink, func) {
      return chainLink(func, nextChainLink);
    }, void 0);

    for (var _len3 = arguments.length, args = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    return chainFunc(args);
  };
}
/**
 * A simple function that performs no action. Useful as a placeholder.
 * @returns {undefined}
 */


function noActionFunc() {
  return void 0;
}
/**
 * Returns a function which lets picking the properties of an object.
 * @param {...string|...number} props An array of properties to pick from the obj.
 * @return {Function} A function which if called picks the revelant properties from its
 * initial argument array and returns a new object with the picked properties.
 */


function pick() {
  for (var _len4 = arguments.length, props = new Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
    props[_key4] = arguments[_key4];
  }

  return function (o) {
    return props.reduce(function (a, e) {
      return _objectSpread(_objectSpread({}, a), {}, _defineProperty({}, e, o[e]));
    }, {});
  };
}

var curryPlaceholderProp = "curryPlaceholderPropRPecoyYmCYqZ2lE";
/**
 * Curry function placeholder.
 *
 * @type {Object}
 */

var _ = _defineProperty({}, curryPlaceholderProp, true);

exports._ = _;
var proceedCallingFnProp = "proceedCallingFnPropRBmGaAgOCgftF9t";
/**
 * Return value of "onFnCall" to call the curried function and return its value.
 *
 * @type {Object}
 */

var proceedCallingFn = _defineProperty({}, proceedCallingFnProp, true);
/**
 * Curries a function.
 * @see https://medium.com/@kj_huang/implementation-of-lodash-curry-function-8b1024d71e3b
 * @param {Function} fn A function to curry in order to return the curried version of the
 * function.
 * @param {Object} [obj] An optional object with further properties to tweak the currying
 * behaviour and execute code while collecting the arguments of the curried function.
 * @param {number|undefined} [obj.arity] The arity of the function, i.e. its number of arguments.
 * If omitted, "fn.length" will be used.
 * @param {Function|undefined} [obj.onEffectiveArgAdded] An optional callback to execute whenever
 * a new effective argument (not a placeholder) is added to the curried function.
 * The callback will receive an object with the following properties as argument:
 * - addedArg: The effective argument added
 * - args: An array with all the previously arguments collected so far without considering "addedArg"
 * - fn: The function "fn";
 * - curriedFn: The current curried function
 * The return value of the function is ignored.
 * @param {Function|undefined} [obj.onPlaceholder] An optional callback to execute whenever a
 * new placeholder is added to the curried function. The callback will receive an object with
 * the following properties as argument:
 * - args: An array with all the previously arguments collected so far without considering "addedArg"
 * - fn: The function "fn"
 * - curriedFn: The current curried function
 * The return value of the function is ignored.
 * @param {Function|undefined} [obj.onFnCall] An optional callback to execute just before
 * calling the "fn" function (i.e. when the "fn" function is ready to be called and all its
 * arguments have been collected). The callback will receive an object with the following properties
 * as argument:
 * - args: The array of the effective arguments of the "fn" function;
 * - fn: The function "fn" which was initially passed to "curry";
 * - curriedFn: The current curried function;
 * This way, the code of the callback may decide what to do and may even call the function on
 * its own and prevent the call from the caller side (i.e. within the "curry" function).
 * The callback must explicitly tell "curry" to call the function by returing "proceedCallingFn".
 * If the callback returns any other value, then "fn" will not be called in "curry" and the
 * return value of this callback will be returned.
 * @param {Function|undefined} [obj.onCurriedFnFirstCall] An optional callback to execute only
 * the first time when the first curried function returned by "curry" is invoked for the very
 * first time with the very first argument or arguments. The callback will receive an object with
 * the following properties as argument:
 * - addedArgs: The arguments provided by the caller
 * - fn: The "fn" function
 * - curriedFn: The current curried function
 * @param {Function|undefined} [obj.onNewCurriedFn] An optional callback to execute initially
 * and each time a new curried function is going to be returned. The callback will an object
 * with the following properties as argument:
 * - curriedFn: The current curried function (same as "newCurriedFn" when this callback is invoked
 * for the very first time);
 * - newCurriedFn: The new curried function (will be the same as "curriedFn" when this callback
 * is invoked for the very first time);
 * @return {Function} The curried version of the function.
 */


exports.proceedCallingFn = proceedCallingFn;

function curry(fn) {
  var _ref2 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref2$arity = _ref2.arity,
      arity = _ref2$arity === void 0 ? void 0 : _ref2$arity,
      _ref2$onEffectiveArgA = _ref2.onEffectiveArgAdded,
      onEffectiveArgAdded = _ref2$onEffectiveArgA === void 0 ? void 0 : _ref2$onEffectiveArgA,
      _ref2$onPlaceholder = _ref2.onPlaceholder,
      onPlaceholder = _ref2$onPlaceholder === void 0 ? void 0 : _ref2$onPlaceholder,
      _ref2$onFnCall = _ref2.onFnCall,
      onFnCall = _ref2$onFnCall === void 0 ? void 0 : _ref2$onFnCall,
      _ref2$onCurriedFnFirs = _ref2.onCurriedFnFirstCall,
      onCurriedFnFirstCall = _ref2$onCurriedFnFirs === void 0 ? void 0 : _ref2$onCurriedFnFirs,
      _ref2$onNewCurriedFn = _ref2.onNewCurriedFn,
      onNewCurriedFn = _ref2$onNewCurriedFn === void 0 ? void 0 : _ref2$onNewCurriedFn;

  var curriedFnFirstCall = true;
  var expectedNumberOfArgs = typeof arity !== "undefined" ? arity : fn.length;
  var nextParameterIndex = 0;
  var placeholdersIndices = [];

  var curriedInner = function curriedInner(expectedNumberOfArgs, nextParameterIndex, placeholdersIndices) {
    for (var _len5 = arguments.length, args = new Array(_len5 > 3 ? _len5 - 3 : 0), _key5 = 3; _key5 < _len5; _key5++) {
      args[_key5 - 3] = arguments[_key5];
    }

    return function curriedFn() {
      var newExpectedNumberOfArgs = expectedNumberOfArgs;
      var newNextParameterIndex = nextParameterIndex;
      var newPlaceholdersIndices = null;
      var argsRequiredChange = false;

      for (var _len6 = arguments.length, addedArgs = new Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
        addedArgs[_key6] = arguments[_key6];
      }

      curriedFnFirstCall && (onCurriedFnFirstCall && onCurriedFnFirstCall({
        addedArgs: addedArgs,
        fn: fn,
        curriedFn: curriedFn
      }) || true) && (curriedFnFirstCall = false);
      var numberOfConsumablePlaceholders = placeholdersIndices.length;

      for (var _i = 0, _addedArgs = addedArgs; _i < _addedArgs.length; _i++) {
        var addedArg = _addedArgs[_i];
        !argsRequiredChange && (args = _toConsumableArray(args));
        argsRequiredChange = true; // "addedArg" may be either a placeholder or an effective argument.

        var isPlaceholder = addedArg ? addedArg[curryPlaceholderProp] === _[curryPlaceholderProp] : false;

        if (numberOfConsumablePlaceholders > 0 && !isPlaceholder) {
          // Argument is an effective argument consuming a previously set placeholder.
          onEffectiveArgAdded && onEffectiveArgAdded({
            addedArg: addedArg,
            args: args,
            fn: fn,
            curriedFn: curriedFn
          });
          var argIndex = void 0;

          if (newPlaceholdersIndices === null) {
            var _placeholdersIndices = _toArray(placeholdersIndices),
                firstIndex = _placeholdersIndices[0],
                rest = _placeholdersIndices.slice(1);

            argIndex = firstIndex;
            newPlaceholdersIndices = rest;
          } else argIndex = newPlaceholdersIndices.shift();

          args[argIndex] = addedArg;
          numberOfConsumablePlaceholders--;
          newExpectedNumberOfArgs--;
        } else {
          if (isPlaceholder) {
            // Argument is a new placeholder.
            if (newPlaceholdersIndices === null) {
              newPlaceholdersIndices = placeholdersIndices.concat(newNextParameterIndex);
            } else newPlaceholdersIndices.push(newNextParameterIndex);

            onPlaceholder && onPlaceholder({
              args: args,
              fn: fn,
              curriedFn: curriedFn
            });
          } else {
            // Argument is an effective argument.
            newExpectedNumberOfArgs--;
            onEffectiveArgAdded && onEffectiveArgAdded({
              addedArg: addedArg,
              args: args,
              fn: fn,
              curriedFn: curriedFn
            });
          }

          args[newNextParameterIndex] = addedArg;
          newNextParameterIndex++;
        }
      }

      return curried.call.apply(curried, [null, curriedFn, newExpectedNumberOfArgs, newNextParameterIndex, newPlaceholdersIndices === null ? _toConsumableArray(placeholdersIndices) : newPlaceholdersIndices].concat(_toConsumableArray(args)));
    };
  };

  var curried = function curried(curriedFn, expectedNumberOfArgs, nextParameterIndex, placeholdersIndices) {
    for (var _len7 = arguments.length, args = new Array(_len7 > 4 ? _len7 - 4 : 0), _key7 = 4; _key7 < _len7; _key7++) {
      args[_key7 - 4] = arguments[_key7];
    }

    if (expectedNumberOfArgs <= 0) {
      if (onFnCall) {
        var shouldCallCurriedFn = onFnCall({
          args: args,
          fn: fn,
          curriedFn: curriedFn
        });

        if (shouldCallCurriedFn && shouldCallCurriedFn[proceedCallingFnProp] === proceedCallingFn[proceedCallingFnProp]) {
          return fn.apply(void 0, args);
        } else return shouldCallCurriedFn;
      } else return fn.apply(void 0, args);
    } else {
      var _newCurried = curriedInner.apply(void 0, [expectedNumberOfArgs, nextParameterIndex, placeholdersIndices].concat(args));

      onNewCurriedFn && onNewCurriedFn({
        curriedFn: curriedFn,
        newCurriedFn: _newCurried
      });
      return _newCurried;
    }
  };

  var newCurried = curriedInner(expectedNumberOfArgs, nextParameterIndex, placeholdersIndices);
  onNewCurriedFn && onNewCurriedFn({
    curriedFn: newCurried,
    newCurriedFn: newCurried
  });
  return newCurried;
}
//# sourceMappingURL=functional.js.map