"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.validatePassword = validatePassword;

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultPasswordSettings = {
  skipDetailedCheck: false,
  contains: {
    upper: true,
    lower: true,
    number: true,
    special: true
  },
  min: 8,
  max: 64
};
/** Validates a password, using the settings passed in to run RegEx tests.
 * @param {string} password The password to validate.
 * @param {object} [settings]
 * @param {boolean} [settings.skipDetailedCheck] - tests the password without running individual
 * checks.
 * @param {boolean} [settings.min] - specify to check for a min length.
 * @param {boolean} [settings.max] - specify to check for a max length.
 * @param {Object} [settings.contains]
 * @param {boolean} [settings.contains.upper] - flag to check for an uppercase character.
 * @param {boolean} [settings.contains.lower] - flag to check for an lowercase character.
 * @param {boolean} [settings.contains.number] - flag to check for a digit.
 * @param {boolean} [settings.contains.special] - flag to check for a special character.
 * @returns {[boolean, object]} An array of fixed length (2) containing the results of each test
 * and if they all passed. [0] - flag for if all regexp tests passed. [1] - an object containing
 * flags for each check specified in settings. */

function validatePassword(password) {
  var settings = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : defaultPasswordSettings;
  settings = _objectSpread(_objectSpread({}, defaultPasswordSettings), settings);
  var validation = {};

  if (password === undefined || password === "") {
    return [false, validation];
  }

  var _settings = settings,
      contains = _settings.contains,
      max = _settings.max,
      min = _settings.min,
      skipDetailedCheck = _settings.skipDetailedCheck;
  var upper = contains.upper,
      lower = contains.lower,
      number = contains.number,
      special = contains.special;
  var length = password.length;
  var lowerRegEx = lower && "(?=.*[a-z])",
      upperRegEx = upper && "(?=.*[A-Z])",
      numberRegEx = number && "(?=.*[\\d])",
      specialRegEx = special && "(?=.*[!@#$%^&*])",
      lenRegEx = min && "{".concat(min, ",").concat(max > 0 ? max : "", "}");

  if (skipDetailedCheck) {
    if (upper && lower && number && special) {
      var _regEx = new RegExp(/(?=.*[a-z])(?=.*[A-Z])(?=.*[\d])(?=.*[!@#$%^&*]){8,64}/);

      var _res = _regEx.test(password);

      return [_res, validation, _regEx];
    } // conditionally build the full regex string.


    var regStr = "";
    if (lowerRegEx) regStr += lowerRegEx;
    if (upperRegEx) regStr += upperRegEx;
    if (numberRegEx) regStr += numberRegEx;
    if (specialRegEx) regStr += specialRegEx;
    if (lenRegEx) regStr += lenRegEx;
    var regEx = new RegExp("^" + regStr);
    var res = regEx.test(regStr);
    return [res, validation, regEx];
  } // validation tests


  if (lower) validation.hasLower = new RegExp(lowerRegEx).test(password);
  if (upper) validation.hasUpper = new RegExp(upperRegEx).test(password);
  if (number) validation.hasNumber = new RegExp(numberRegEx).test(password);
  if (special) validation.hasSpecial = new RegExp(specialRegEx).test(password);
  if (min) validation.correctLength = length > min;
  if (max) validation.correctLength = length < max; // true if all entries in vResults is true.

  var success = Object.entries(validation).every(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        _ = _ref2[0],
        val = _ref2[1];

    return val === true;
  });
  return [success, validation];
}
//# sourceMappingURL=validation.js.map