"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = declarativeFactory;

var _introspection = require("../object-handling/introspection");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * A function to easily create interchangeable factories switching them
 * in a declarative way based on the given runtime parameters.
 * @param {Array<[Function, *]>} tuples An array of tuples, each tuple being an array of two elements:
 *  - testCondition: A boolean value or a function returning a boolean value which, if true,
 * instructs this function to return the factory bound to the test condition.
 * Note: the first factory for which the test condition is truthy will be returned and further
 * tuples will be ignored;
 * - factory: Anything. The value to return if testCondition is a truthy value
 * or a function returning a truthy value.
 * The last element of the returned tuples MAY not be a tuple array with two elements,
 * and in such case it will be treated as the default factory value to return if none
 * of the test conditions of the previous factories are satisfied.
 * If the last tuple is not a default value and none of the test conditions of the previous
 * factories are satisfied as well as for the factory of last tuple, then null
 * will be returned by this function.
 * @return {*|null} The first factory value for which the test is truthy or returns a truthy
 * value, a default factory value, or null.
 */
function declarativeFactory(tuples) {
  var isConditionMet = function isConditionMet(condition) {
    return Boolean(typeof condition === "function" ? condition() : condition);
  };

  var factoryValue = function () {
    var i = 0;

    for (; i < tuples.length - 1; i++) {
      // handle all tuples except the last
      var tuple = tuples[i];

      var _tuple = _slicedToArray(tuple, 2),
          condition = _tuple[0],
          factoryVal = _tuple[1];

      if (isConditionMet(condition)) return factoryVal; // is condition satified?
    }

    var lastTuple = tuples[i];

    if ((0, _introspection.isArray)(lastTuple) && lastTuple.length === 2) {
      // handle last
      var _lastTuple = _slicedToArray(lastTuple, 2),
          _condition = _lastTuple[0],
          _factoryVal = _lastTuple[1];

      if (isConditionMet(_condition)) return _factoryVal; // is last condition satified?

      return null; // No default and no factory value satisfying a test condition.
    } else return lastTuple; // Default factory value.

  }();

  return factoryValue;
}
//# sourceMappingURL=declarative-factory.js.map