"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createEnum = createEnum;
exports.deepFreeze = deepFreeze;

function _typeof(obj) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (obj) { return typeof obj; } : function (obj) { return obj && "function" == typeof Symbol && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }, _typeof(obj); }

/**
 * Deep freezes an object using recursion.
 * @param {Object} obj the object to freeze.
 * @returns {Object} A reference to the frozen object. */
function deepFreeze(obj) {
  if (_typeof(obj) !== "object") return obj;

  for (var key in obj) {
    var val = obj[key];
    if (_typeof(val) === "object") deepFreeze(val);
  }

  return Object.freeze(obj);
}
/**
 * Create an enum-like object from a list of values. The enum will be frozen to prevent
 * further modification.
 * @param  {...any} values 
 * @returns The enum.
 */


function createEnum() {
  var enumObj = {};

  for (var _len = arguments.length, values = new Array(_len), _key = 0; _key < _len; _key++) {
    values[_key] = arguments[_key];
  }

  for (var val in values) {
    enumObj[val] = val;
  }

  return deepFreeze(enumObj);
}
//# sourceMappingURL=immutability.js.map