"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.arrToObject = arrToObject;
exports.completeObjectAssign = completeObjectAssign;
exports.defineProperty = defineProperty;
exports.getPrototypes = getPrototypes;
exports.mapObject = mapObject;
exports.prop = prop;
exports.selectWhere = selectWhere;

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _iterableToArrayLimit(arr, i) { var _i = arr == null ? null : typeof Symbol !== "undefined" && arr[Symbol.iterator] || arr["@@iterator"]; if (_i == null) return; var _arr = []; var _n = true; var _d = false; var _s, _e; try { for (_i = _i.call(arr); !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

/**
 * Maps an object, executing a function on each of its properties
 * returning a new mapped object.
 * @param {Object} obj The object to map.
 * @param {Function} func The function to use for the mapping.
 * @return {Object} The new mapped object.
 */
function mapObject(obj, func) {
  return Object.fromEntries(Object.entries(obj).map(function (_ref, i) {
    var _ref2 = _slicedToArray(_ref, 2),
        k = _ref2[0],
        v = _ref2[1];

    return [k, func(v, k, i)];
  }));
}
/**
 * Maps an array to an object.
 * @param {Array} arr An array.
 * @param {Function} func A function receiving the current value, its index as well as the whole
 * array "arr" as parameters returning a tuple with the key at index 0 and the value at index 1
 * to set on the object.
 * @returns {Object} The array mapped to an object.
 */


function arrToObject(arr, func) {
  var res;
  return arr.reduce(function (carry) {
    for (var _len = arguments.length, rest = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      rest[_key - 1] = arguments[_key];
    }

    return (res = func.apply(void 0, rest)) && false || (carry[res[0]] = res[1]) && false || carry;
  }, {});
}
/**
 * Selects the properties of an object which return true for a
 * supplied callback function.
 * @param {Object} obj The input object.
 * @param {Function} func A function which will receive two parameters:
 * - value: The current value of the input object for a given property;
 * - prop: The name of the current property of the input object.
 * and return true or false based on the
 * @return {Object} An object having only the properties which return true for
 * the callback function.
 */


function selectWhere(obj, func) {
  return Object.keys(obj).reduce(function (carry, key) {
    if (func(obj[key], key)) carry[key] = obj[key];
    return carry;
  }, {});
}
/**
 * Returns all the properties of the given object traversing its prototype chain.
 * @param {Object} obj The object.
 * @param {Object} [options] An object with options.
 * @param {?Object} [options.stopAt] The prototype in the chain at which to stop
 * the traversing. Defaults to null, in which case the whole prototype chain will be traversed.
 * @param {boolean} [options.includeStop] Whether or not to include the properties of the given
 * prototype at which to stop. Works only if "stopAtPrototype" is set to a valid prototype object
 * in the prototype chain of "obj". Defaults to true, in which case the properties of the prototype
 * at which to stop will be included in the returned array.
 * @return {string[]} An array of containing the names of the properties.
 */


function getPrototypes(obj) {
  var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref3$stopAt = _ref3.stopAt,
      stopAt = _ref3$stopAt === void 0 ? null : _ref3$stopAt,
      _ref3$includeStop = _ref3.includeStop,
      includeStop = _ref3$includeStop === void 0 ? true : _ref3$includeStop;

  var current = obj;
  var map = {};
  var stopAtFound = false;

  while (current && (!stopAtFound || includeStop)) {
    var _properties = Object.getOwnPropertyNames(current);

    _properties.map(function (property) {
      if (stopAtFound) return;
      if (stopAt !== property || includeStop) map[property] = true;
      if (stopAt === property) stopAtFound = true;
    });

    if (stopAtFound) break;
    current = Object.getPrototypeOf(current);
    stopAtFound = stopAt && current === stopAt;
  }

  var properties = Object.keys(map);
  return properties;
}
/**
 * Defines an object's property with a getter and an optional setter.
 *
 * @param {Object} obj An object (may be a prototype).
 * @param {string} propName The property name.
 * @param {Function} getFunc Getter function.
 * @param {Function|undefined} [setFunc] Setter function.
 * @return {undefined}
 */


function prop(obj, propName, getFunc) {
  var setFunc = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : void 0;
  var propObj = {};
  propObj[propName] = {
    get: getFunc,
    set: setFunc
  };
  Object.defineProperties(obj, propObj);
}
/**
 * Defines a property on an object.
 * @param {Object} o An object.
 * @param {string} p The property to define.
 * @param {Object} descriptor Optional object containing the descriptor's properties
 * to use to override the default properties.
 * @return {Object} The object that was passed to the function.
 */


function defineProperty(o, p) {
  var descriptor = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  return Object.defineProperty(o, p, _objectSpread({}, descriptor));
}
/**
 * Assigns the properties of the given source objects to the target object.
 * @source https://stackoverflow.com/questions/39515321/spread-operator-issues-with-property-accessors-getters#answer-39521418
 * @param {Object} target The target object.
 * @param {...Object} sources The source objects.
 * @return {Object} The target object.
 */


function completeObjectAssign(target) {
  for (var _len2 = arguments.length, sources = new Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
    sources[_key2 - 1] = arguments[_key2];
  }

  sources.forEach(function (source) {
    var descriptors = Object.keys(source).reduce(function (descriptors, key) {
      descriptors[key] = Object.getOwnPropertyDescriptor(source, key);
      return descriptors;
    }, {}); // By default, Object.assign copies enumerable Symbols too.

    Object.getOwnPropertySymbols(source).forEach(function (sym) {
      var descriptor = Object.getOwnPropertyDescriptor(source, sym);

      if (descriptor.enumerable) {
        descriptors[sym] = descriptor;
      }
    });
    Object.defineProperties(target, descriptors);
  });
  return target;
}
//# sourceMappingURL=objects.js.map