"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _CTypeError = _interopRequireDefault(require("../errors/CTypeError"));

var _availableRegex = _interopRequireDefault(require("./available-regex"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it["return"] != null) it["return"](); } finally { if (didErr) throw err; } } }; }

function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }

function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); Object.defineProperty(Constructor, "prototype", { writable: false }); return Constructor; }

/**
 * A simple helper class for creating new regex instances. By default, strings are
 * escaped before being passed to a RegExp constructor. This class also exposes common RegExp
 * utilities, such as testing emails and passwords.
 */
var RegExpHelper = /*#__PURE__*/function () {
  function RegExpHelper() {
    _classCallCheck(this, RegExpHelper);
  }

  _createClass(RegExpHelper, null, [{
    key: "filterInt",
    value:
    /**
     * Filter numbers from a string.
     * @param {string} str the string to filter.
     * @returns The modified string.
     */
    function filterInt(str) {
      var filtered = str.replace(/[^0-9]/g, "");
      filtered = filtered.replace(/^[0]+([1-9])/, "$1");
      filtered = filtered.replace(/^[0]+$/, "0");

      if (str && filtered.length && str[0] === "-") {
        filtered = "-".concat(filtered);
      }

      return filtered;
    }
    /**
     * Filter decimal numbers from a string.
     * @param {string} str the string to filter.
     * @returns The modified string.
     */

  }, {
    key: "filterFloat",
    value: function filterFloat(str) {
      var filtered = str.replace(/[^0-9.]/g, "");
      var regex = /(\..*)\./g;
      var replace = "$1";

      do {
        filtered = filtered.replace(regex, replace);
      } while (filtered != filtered.replace(regex, replace));

      filtered === "." ? filtered = "0." : filtered;
      filtered = filtered.replace(/^[0]+([1-9])/, "$1");
      filtered = filtered.replace(/^[0]+($|\.)/, "0$1");
      if (str && filtered.length && str[0] === "-") filtered = "-".concat(filtered);
      return filtered;
    }
    /**
     * Create a new RegEx from a string. Escapes the RegExp string before converting.
     * @param {string} value The value to escape and convert to a RegExp.
     * @param {string} [flags] Optional flags to pass to the RegExp.
     * @returns {RegExp} The new RegExp instance.
     */

  }, {
    key: "fromString",
    value: function fromString(value, flags) {
      if (typeof value !== "string") throw new _CTypeError["default"]("string", value);
      return new RegExp(this.escapeString(value), flags);
    }
    /**
     * Escape a RegExp string. Escapes characters with special meaning either inside
     * or outside character sets. Uses a simple backslash escape when it’s always valid, and a
     * `\xnn` escape when the simpler form would be disallowed by Unicode patterns’ stricter
     * grammar.
     * @param {string} value
     * @returns The escaped string.
     */

  }, {
    key: "escapeString",
    value: function escapeString(value) {
      if (typeof value !== "string") throw new _CTypeError["default"]("string", value);
      return value.replace(/[|\\{}()[\]^$+*?.]/g, "\\$&").replace(/-/g, "\\x2d");
    }
    /**
     * Get a new instance of one of the 'built-in' RegExp's this class offers.
     * Common RegExp's include email and password. See `getAvailableRegExp()` for the full list.
     * @param {string} name The name of the RegExp.
     * @param {Object} options Extra options used for configuring the RegExp.
     * @param {string} [options.flags] Optional flags to pass to the new RegExp. If the built-in
     * RegExp does not support additional flags being passed, they will be ignored.
     * @returns {RegExp | null} The new instance, or null if the name was not valid.
     */

  }, {
    key: "useCommonRegExp",
    value: function useCommonRegExp(name, options) {
      options = options || {};
      var regInfo;
      if (name in _availableRegex["default"]) regInfo = _availableRegex["default"][name];else return null;
      if (!regInfo.allowChange) return new RegExp(regInfo.source);
      return new RegExp(regInfo.source, options.flags);
    }
    /**
     * Get info on a built-in RegExp.
     * @param {string} name
     * @returns {Object} an object containing name, description, whether flags can be passed when
     * creating a new instance and possible examples for ease of use.
     */

  }, {
    key: "getCommonRegExpInfo",
    value: function getCommonRegExpInfo(name) {
      var regInfo;
      if (name in _availableRegex["default"]) regInfo = _availableRegex["default"][name];else return null;
      var _regInfo = regInfo,
          allowChange = _regInfo.allowChange,
          description = _regInfo.description,
          displayName = _regInfo.displayName,
          examples = _regInfo.examples;
      return {
        displayName: displayName,
        description: description,
        allowChange: allowChange,
        examples: examples
      };
    }
    /** Get a list of the currently available built-in RegExp.
     * @returns An array of tuples, with the first value being the property name and the
     * second being the info about the RegExp.
     */

  }, {
    key: "getAvailableRegExp",
    value: function getAvailableRegExp() {
      var infoList = [];

      var _iterator = _createForOfIteratorHelper(_availableRegex["default"]),
          _step;

      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var key = _step.value;
          infoList.push([key, _availableRegex["default"][key]]);
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }

      return infoList;
    }
  }]);

  return RegExpHelper;
}();

exports["default"] = RegExpHelper;
//# sourceMappingURL=RegExpHelper.js.map