"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getDirection = getDirection;
exports.getHorizontalDirection = getHorizontalDirection;
exports.getPointerCoordinates = getPointerCoordinates;
exports.getVerticalDirection = getVerticalDirection;

/**
 * Takes a mouse or a touch events and returns clientX and clientY values
 * @param event
 * @return {[number, number]}
 */
function getPointerCoordinates(event) {
  if (event.touches) {
    var _event$touches$ = event.touches[0],
        _clientX = _event$touches$.clientX,
        _clientY = _event$touches$.clientY;
    return [_clientX, _clientY];
  }

  var clientX = event.clientX,
      clientY = event.clientY;
  return [clientX, clientY];
}
/**
 * Get the direction of a horizontal movement.
 * @returns {"right" | "left"}
 */


function getHorizontalDirection(alpha) {
  return alpha < 0 ? "right" : "left";
}
/**
 * Get the direction of a vertical movement.
 * @returns {"down" | "up"}
 */


function getVerticalDirection(alpha) {
  return alpha < 0 ? "down" : "up";
}
/**
 * Get the direction of a movement.
 * @returns {"down" | "up" | "right" | "left"}
 */


function getDirection(currentPoint, startingPoint, alpha) {
  var alphaX = startingPoint[0] - currentPoint[0];
  var alphaY = startingPoint[1] - currentPoint[1];
  if (Math.abs(alphaX) > Math.abs(alphaY)) return getHorizontalDirection(alpha[0]);
  return getVerticalDirection(alpha[1]);
}
//# sourceMappingURL=swipe.js.map